﻿LootMaster          = LibStub("AceAddon-3.0"):NewAddon("CouncilLootMaster", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")

local version 	    			= "0.3"
local iVersion	    			= 3
local lastCompatibleVersion 	= 3
local _G            			= _G

local debug         = false
local addon         = LootMaster		-- Local instance of the addon

local L = LibStub("AceLocale-3.0"):GetLocale("CouncilLootMaster")

local usage = [=[

This mod provides a full loot council system based on some submodules from the EPGPLootMaster addon.
USAGE: Create a party/raid, setup a master looter. When master loot drops, just click the item and the UI will open.
Commandline Tools:
/cloot version: Show the versionchecker
/cloot config: Show the options window
/cloot reset: Resets window locations and scale
/cloot hide: Manually hide the Master Looter UI
/cloot show: Manually show the Master Looter UI
/cloot toggle: Manually toggle between showing and hiding the Master Looter UI
]=]

--[[
    Returns a table serialized as a string.
]]
function tprint( data, level )
	level = level or 0
	local ident=strrep('    ', level)
	if level>5 then return end

	if type(data)~='table' then print(tostring(data)) end;

	for index,value in pairs(data) do repeat
		if type(value)~='table' then
			print( ident .. '['..index..'] = ' .. tostring(value) .. ' (' .. type(value) .. ')' );
			break;
		end
		print( ident .. '['..index..'] = {')
        tprint(value, level+1)
        print( ident .. '}' );
	until true end
end

function LootMaster:GetVersionString()
    return self.version;
end

function LootMaster:OnInitialize()

    self.version 	= version;
    self.iVersion 	= iVersion;

    self.db = LibStub("AceDB-3.0"):New("CouncilLootMaster")

    self.db:RegisterDefaults(
    {
      profile = {
        
        auto_announce_threshold = 4,
        AutoLootThreshold = 2,
        hideOnSelection = true,
        loot_timeout = 60,
        filterCouncilLootmasterMessages = true,

		voting = true,
		votingDisallowSelf = false,
		votingSendGuildRank = true,
		votingSendGuildRankList = {true},
        votingSendAssistantOnly = false,

        monitor = true,
        monitorSend = true,
		monitorSendGuildRank = true,
		monitorSendGuildRankList = {true},
        monitorSendAssistantOnly = false,
        monitorThreshold = 2,
        ignoreResponseCorrections = false,
        hideMLOnCombat = false,
        hideSelectionOnCombat = false,
        allowCandidateNotes = true,
        monitorIncomingThreshold = 3,
        audioWarningOnSelection = true,
        use_councillootmaster = 'ask'
      }
    })

    self.lootList   = {};
    self.lootMLCache = {};

    -- Client responses, DO NOT CHANGE ORDER!
    self.RESPONSE = {
        { ["CODE"]      = "NOTANNOUNCED",   ["SORT"] =  100,  ["COLOR"] = {1,1,1},        ["TEXT"] = L['Not yet announced to candidate'] },
        { ["CODE"]      = "INIT",           ["SORT"] =  200,  ["COLOR"] = {1,0,0},        ["TEXT"] = L['Offline or lootmaster not installed?'] },
        { ["CODE"]      = "WAIT",           ["SORT"] =  300,  ["COLOR"] = {1,0.5,0},      ["TEXT"] = L['Making selection, please wait...'] },
        { ["CODE"]      = "TIMEOUT",        ["SORT"] =  400,  ["COLOR"] = {1,0,1},        ["TEXT"] = L['Candidate did not respond on time.'] },
        { ["CODE"]      = "MAIN",           ["SORT"] =  500,  ["COLOR"] = {0,1,0},    	  ["TEXT"] = 'Mainspec' },
        { ["CODE"]      = "OFF",          	["SORT"] =  600,  ["COLOR"] = {1,0.65,0},    ["TEXT"] = 'Offspec' },
        { ["CODE"]      = "PASS",           ["SORT"] = 	700,  ["COLOR"] = {0.6,0.6,0.6},  ["TEXT"] = L['Pass'] }
    }
    for i,d in ipairs(self.RESPONSE) do
        self.RESPONSE[d.CODE] = i;
        self.RESPONSE[d.CODE .. "_TEXT"] = d.TEXT;
    end
    LootMaster.RESPONSE = self.RESPONSE;

    -- Loot receive types.
    self.LOOTTYPE = {
        { ["CODE"]      = "UNKNOWN",        ["TEXT"] = L['%s received %s for unknown reason%4$s.'] },
        { ["CODE"]      = "VOTE",           ["TEXT"] = L['%s received %s.'] },
        { ["CODE"]      = "DISENCHANT",     ["TEXT"] = L['%s received %s for disenchantment%4$s.'] }
    }
    for i,d in ipairs(self.LOOTTYPE) do
        self.LOOTTYPE[d.CODE] = i;
        self.LOOTTYPE[d.CODE .. "_TEXT"] = d.TEXT;
    end
    LootMaster.LOOTTYPE = self.LOOTTYPE;

    -- Register communications
    self:RegisterComm("CLootMasterC",	"CommandReceived")

      -- Register communications for version checking
    self:RegisterComm("CouncilLMVChk",			"CommVersionCheckRequest")
	self:RegisterComm("CouncilLMVRsp",			"CommVersionCheckResponse")

      -- Check for updates versions in the guild
	if IsInGuild() then
		--self:SendCommMessage("CouncilLMVChk",      lastCompatibleVersion .. "_" .. iVersion .. "_" .. version, "GUILD")
	end

    self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat");
    self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat");

	self:RegisterEvent("PLAYER_ENTERING_WORLD", "EnterWorld");

    self:RegisterChatCommand("cloot",	        "SlashHandler")
end

function LootMaster:EnterWorld()
	RegisterAddonMessagePrefix("CLootMasterC")
	RegisterAddonMessagePrefix("CouncilLMVChk")
	RegisterAddonMessagePrefix("CouncilLMVRsp")
	RegisterAddonMessagePrefix("CLootMasterML")
end

function LootMaster:OnEnable()
    -- We don't need these right away, so localize them after a 10 second delay.
    self:ScheduleTimer("LocalizeLootTypes", 10)
end

-- Preparation to fix the portal "bug" in naxx when handing out loot to players that have already
-- used to portal.
local function EmulateLocal_CHAT_MSG_LOOT_proc(player, item, ...)
    for i = 1, select( "#", ... ) do
        local frame = select( i, ... )
        local func = frame:GetScript('OnEvent');
        pcall( func, frame, 'CHAT_MSG_LOOT', format(LOOT_ITEM, tostring(player), tostring(item)), '', '', '', '' )
    end
end
local function EmulateLocal_CHAT_MSG_LOOT( player, item )
    EmulateLocal_CHAT_MSG_LOOT_proc( player, item, GetFramesRegisteredForEvent('CHAT_MSG_LOOT') )
end

function LootMaster:SlashHandler( input )
	local _,_,command, args = string.find( input, "^(%a-) (.*)$" )
	command = command or input

	if command=='version' or command=='versioncheck' or command=='vc' or command=='v' then

        self:ShowVersionCheckFrame();

	elseif command=='debug' then

        self.debug = not self.debug;
        if self.debug then
            self:Print('Debugging enabled')
        else
            self:Print('Debugging disabled')
        end

    elseif command=='verbose' then

        self.verbose = not self.verbose;
        if self.verbose then
            self.debug = true;
            self:Print(L['Verbose debugging enabled'])
        else
            self.debug = false;
            self:Print(L['Verbose debugging disabled'])
        end

    elseif command=='reset' then

        LootMaster:SetUIScale(1.0);
        if LootMasterML then
            LootMasterML:SetUIScale(1.0)
        end

    elseif command=='config' or command=='c' or command=='options' or command=='o' then

        InterfaceOptionsFrame_OpenToCategory("CouncilLootMaster")

    elseif command=='close' or command=='hide' then

        if LootMasterML then
            self:Print('Hiding lootmaster window, reopen with /cloot show')
            LootMasterML.Hide(LootMasterML)
        end

    elseif command=='open' or command=='show' then

        if LootMasterML then
            LootMasterML.Show(LootMasterML)
        end

    elseif command=='toggle' or command=='toggel' then

        if LootMasterML then
            if LootMasterML.IsShown(LootMasterML) then
                LootMasterML.Hide(LootMasterML)
            else
                LootMasterML.Show(LootMasterML)
            end
        end

    elseif command=='emulate' then

        local player, item = strmatch(strtrim(args or ''), '(%S+)%s+(.+)');
        if not player or not item or player=='' or item=='' then
            self:Print('Usage: /cloot emulate player [itemlink]')
            self:Print('This will emulate the "Player receives [item]." locally. Usually used to "fix" the portal problem in naxx.')
        else
            EmulateLocal_CHAT_MSG_LOOT( player, item )
        end

    elseif command=='add' or command=='announce' then

        if not LootMasterML then return self:Print(L['Could not add loot, master looter module not active']) end
        ml = LootMasterML;

        local lootLink = strtrim(args or '');
        if not args or not lootLink or lootLink=='' then return self:Print(format('Usage: /cloot %s [lootlink]', command)) end;

        local loot = ml.GetLoot(ml, lootLink);
        local added = false
        if not loot then
            local lootID = ml.AddLoot(ml, lootLink, true);
            loot = ml.GetLoot(ml, lootID);
            loot.announced = false;
            loot.manual = true;
            added = true;
        end
        if not loot then return self:Print(L['Unable to register loot.']) end;

        local num = GetNumGroupMembers();
        local name = nil;
        if num>0 then
            -- we're in raid
            for i=1, num do
                name = GetRaidRosterInfo(i)
				local playerLocalClass,playerClass = UnitClass(name)
				if self:EligibleItemCheck( lootLink, playerLocalClass, playerClass ) then
					ml.AddCandidate(ml, loot.id, name)
				end
            end
        else
            num = GetNumSubgroupMembers()
            for i=1, num do
                name = UnitName('party'..i)
				local playerLocalClass,playerClass = UnitClass(name)
				if self:EligibleItemCheck( lootLink, playerLocalClass, playerClass ) then
					ml.AddCandidate(ml, loot.id, name)
				end
            end
			local playerLocalClass, playerClass = UnitClass('player')
			if self:EligibleItemCheck( lootLink, playerLocalClass, playerClass ) then
				ml.AddCandidate(ml, loot.id, name)
			end
        end

        if command=='announce' then
            ml.AnnounceLoot(ml, loot.id)
        end

        if added then
            ml.SendCandidateListToMonitors(ml, loot.id)
        end

        ml.ReloadMLTableForLoot( ml, loot.link )

    elseif command=='debuglog' or command=='log' or command=='lootlog' then

        self:OutputLog();

    elseif command=='logtest' then

        for i = 1, 20 do
            local item = GetInventoryItemLink("player",i);


            if item then
                local itemName = tostring(GetItemInfo(item));
                local itemID = tostring(LootMasterML:GetItemIDFromLink(item))
                local entry = self:CreateLogEntry();
                entry.name=itemID..':'..itemName
                entry.slots={}
                for j = 1, 20 do
                    local litem = GetInventoryItemLink("player",j);

                    if litem then
                        local litemName = tostring(GetItemInfo(litem));
                        local litemID = tostring(LootMasterML:GetItemIDFromLink(litem))
                        entry.slots[j] = litemID..':'..litemName
                    else
                        entry.slots[j] = 'empty'
                    end

                end
            end
        end

        self:OutputLog();
        self:Print('done');

	else

		self:Print( format('%s loaded.', version) .. usage )

	end
end

function LootMaster:ColorHexToRGB(color)
    color = tostring(color)
    local r,g,b = strmatch(color,'(%x%x)(%x%x)(%x%x)')
    r = tonumber(format('0x%s', r or 'ff'))/255
    g = tonumber(format('0x%s', g or 'ff'))/255
    b = tonumber(format('0x%s', b or 'ff'))/255
    return r,g,b
end

function LootMaster:ColorRGBToHex(r,g,b)
    r = tonumber(r) or 1
    g = tonumber(g) or 1
    b = tonumber(b) or 1
    return format('%02x%02x%02x',floor(r*255), floor(g*255), floor(b*255))
end

--[[
    Data for the GetGearByINVTYPE function
]]--
local INVTYPE_Slots = {
		INVTYPE_HEAD		    = "HeadSlot",
		INVTYPE_NECK		    = "NeckSlot",
		INVTYPE_SHOULDER	    = "ShoulderSlot",
		INVTYPE_CLOAK		    = "BackSlot",
		INVTYPE_CHEST		    = "ChestSlot",
		INVTYPE_WRIST		    = "WristSlot",
		INVTYPE_HAND		    = "HandsSlot",
		INVTYPE_WAIST		    = "WaistSlot",
		INVTYPE_LEGS		    = "LegsSlot",
		INVTYPE_FEET		    = "FeetSlot",
		INVTYPE_SHIELD		    = "SecondaryHandSlot",
		INVTYPE_ROBE		    = "ChestSlot",
		INVTYPE_2HWEAPON	    = {"MainHandSlot","SecondaryHandSlot"},
		INVTYPE_WEAPONMAINHAND	= "MainHandSlot",
		INVTYPE_WEAPONOFFHAND	= {"SecondaryHandSlot",["or"] = "MainHandSlot"},
		INVTYPE_WEAPON		    = {"MainHandSlot","SecondaryHandSlot"},
		INVTYPE_THROWN		    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_RANGED		    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_RANGEDRIGHT 	= {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_FINGER		    = {"Finger0Slot","Finger1Slot"},
		INVTYPE_HOLDABLE	    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_TRINKET		    = {"TRINKET0SLOT", "TRINKET1SLOT"}
}

--[[ Extract the itemlinks, itemlevel and texture of the players current equipment for
    the given inventory slot.
]]--
function LootMaster:GetGearByINVTYPE( INVTYPE, unit )

    if not unit then unit="player" end

	if not INVTYPE_Slots[INVTYPE] then return '' end;
	local ret = {}
	local slot = INVTYPE_Slots[INVTYPE];

	local item = GetInventoryItemLink(unit,GetInventorySlotInfo(slot[1] or slot))
	if not item and slot['or'] then
		item = GetInventoryItemLink(unit,GetInventorySlotInfo(slot['or']))
	end;
	if item then tinsert(ret, item) end;
	if slot[2] then
		item = GetInventoryItemLink(unit,GetInventorySlotInfo(slot[2]))
		if item then tinsert(ret, item) end;
	end
    for i, item in ipairs(ret) do
        local _, _, _, ilevel, _, _, _, _, _, itemTexture = GetItemInfo(item)
        ret[i] = format('%s^%s^%s', item, ilevel or -1, itemTexture)
    end
	return strjoin('$', unpack(ret));
end

-- Try to extract BoP, BoE or BoU status for item
function LootMaster:GetItemBinding(item)
	if not item then return end
	if not self.bindingtooltip then
		self.bindingtooltip = CreateFrame("GameTooltip", "LootMasterBindingTooltip", UIParent, "GameTooltipTemplate")
	end
	local tip = self.bindingtooltip
	tip:SetOwner(UIParent, "ANCHOR_NONE")
	tip:SetHyperlink(item)
  local numLines = LootMasterBindingTooltip:NumLines()
  if numLines>6 then numLines=6 end
  for i=1, numLines do
    local line = _G['LootMasterBindingTooltipTextLeft' .. i]
    if line and line.GetText then
      local t = line:GetText()
      if t == ITEM_BIND_ON_PICKUP then
        tip:Hide()
        return "pickup"
      elseif t == ITEM_BIND_ON_EQUIP then
        tip:Hide()
        return "equip"
      elseif t == ITEM_BIND_ON_USE then
        tip:Hide()
        return "use"
      end
    end
  end
	tip:Hide()
	return nil
end

-- Default english locale, this will automatically get updated by the
-- UpdateClassTranslator function, called from other places in the CouncilLM package.
local classLocalizeTable = {
    ['MAGE']            = 'Mage',
    ['WARRIOR']         = 'Warrior',
    ['DEATHKNIGHT']     = 'Death Knight',
    ['WARLOCK']         = 'Warlock',
    ['DRUID']           = 'Druid',
    ['SHAMAN']          = 'Shaman',
    ['ROGUE']           = 'Rogue',
    ['PRIEST']          = 'Priest',
    ['PALADIN']         = 'Paladin',
    ['HUNTER']          = 'Hunter',
	['MONK']       		= 'Monk'
}
local classUnlocalizeTable = {};
-- Build the reverse lookup table.
for u, l in pairs(classLocalizeTable) do
    classUnlocalizeTable[l] = u
end

-- Table of eligible classes for each loot type (bar offhands)
local eligibleTable = {
	['One-Handed Axes']     = {'DEATHKNIGHT','PALADIN','SHAMAN','WARRIOR','ROGUE','MONK'},
    ['One-Handed Maces']    = {'ROGUE','PRIEST','SHAMAN','DRUID','DEATHKNIGHT','PALADIN','WARRIOR'},
    ['One-Handed Swords']   = {'DEATHKNIGHT','PALADIN','WARLOCK','MAGE','MONK','ROGUE','WARRIOR'},
    ['Daggers']             = {'DRUID','MAGE','PRIEST','ROGUE','SHAMAN','WARLOCK'},
    ['Fist Weapons']        = {'DRUID','MONK','ROGUE','SHAMAN','WARRIOR'},
    ['Wands']               = {'MAGE','PRIEST','WARLOCK'},
    ['Two-Handed Axes']     = {'DEATHKNIGHT','PALADIN','SHAMAN','WARRIOR'},
    ['Two-Handed Maces']    = {'DEATHKNIGHT','DRUID','PALADIN','SHAMAN','WARRIOR'},
    ['Two-Handed Swords']   = {'DEATHKNIGHT','PALADIN','WARRIOR'},
    ['Polearms']            = {'DEATHKNIGHT','MONK','PALADIN','WARRIOR','DRUID'},
    ['Staves']              = {'DRUID','MAGE','MONK','PRIEST','SHAMAN','WARLOCK','WARRIOR'},
    ['Bows']                = {'HUNTER'},
	['Crossbows']           = {'HUNTER'},
    ['Guns']                = {'HUNTER'},
    ['Shields']             = {'WARRIOR','PALADIN','SHAMAN'},
	['Cloth']				= {'MAGE','WARLOCK','PRIEST'},
    ['Leather']             = {'ROGUE','DRUID','MONK'},
    ['Mail']                = {'HUNTER','SHAMAN'},
    ['Plate']               = {'DEATHKNIGHT','PALADIN','WARRIOR'}
}

-- Separate list of offhands since they dont have their own itemSubType
local eligibleForOffhands = {'DRUID','MAGE','MONK','PALADIN','PRIEST','SHAMAN','WARLOCK'}

-- Make the lookup table for localized subTypes.
local subTypeLocalized = {}
for l, _ in pairs(eligibleTable) do subTypeLocalized[l]=l end;



function LootMaster:EligibleItemCheck( itemLink, playerLocalClass, playerClass )
	if not itemLink then return end	
	
    if not self.bindingtooltip then
      self.bindingtooltip = CreateFrame("GameTooltip", "LootMasterBindingTooltip", UIParent, "GameTooltipTemplate")
    end
    local tip = self.bindingtooltip
    tip:SetOwner(UIParent, "ANCHOR_NONE")
    tip:SetHyperlink(itemLink)

    -- lets see if we can find a 'Classes: Mage, Druid' string on the itemtooltip
    -- just scan all the lines.
    for i = 1, LootMasterBindingTooltip:NumLines() do
      local linetext = _G["LootMasterBindingTooltipTextLeft" .. i]
      local text = linetext:GetText()
      local localizedClasses = gsub( text or '', ', ', ',' )
      localizedClasses = localizedClasses:match( gsub(ITEM_CLASSES_ALLOWED,"%%s","(.*)") )

      if localizedClasses and string.find(localizedClasses,playerLocalClass) then
        -- Yep, this item is available for this class
        tip:Hide()
		return true
	  end
    end

    tip:Hide()
	
	local _, _, _, _, _, _, itemSubType, _, itemEquipLoc = GetItemInfo(itemLink)
	
	if itemEquipLoc and itemEquipLoc==INVTYPE_HOLDABLE then
		for _,offhClass in ipairs(eligibleForOffhands) do
			if offhClass==playerClass then
				return true
			end
		end
	elseif itemEquipLoc and (itemEquipLoc==INVTYPE_NECK or itemEquiploc==INVTYPE_BODY or itemEquiploc==INVTYPE_FINGER or itemEquiploc==INVTYPE_TRINKET or itemEquiploc==INVTYPE_CLOAK or itemEquiploc==INVTYPE_TABARD) then
		return true
	elseif itemSubType then
		local eligibleClassArray = eligibleTable[ subTypeLocalized[itemSubType] ]
        if eligibleClassArray then
            -- There is a list of classes for this item type
            for _, class in ipairs(eligibleClassArray) do
                if class==playerClass then
					return true
				end
            end
        end
	end
	return false
end

-- localize a class by using cached strings.
-- example LocalizeClass( 'DEATHKNIGHT') returns 'Death Knight'.
function LootMaster:LocalizeClass( classFilename )
    return classLocalizeTable[classFilename]
end

-- unlocalize a class by using cached strings.
-- example UnlocalizeClass( 'Death Knight' ) returns 'DEATHKNIGHT'.
function LootMaster:UnlocalizeClass( localizedClass )
    return classUnlocalizeTable[localizedClass]
end

-- A simple function to update the class translation tables.
function LootMaster:UpdateClassLocalizer( localizedClass, classFilename )
    classLocalizeTable[classFilename] = localizedClass
    classUnlocalizeTable[localizedClass] = classFilename
end

-- This function tries to localize the itemSubTypes used in GetItemAutoPassClasses()
-- It tries to do so by looking up a few known items.
local subTypeLocalizedLookup = {
    ['One-Handed Axes']     = 'Hitem:31071',    -- Grom'tor's Charge
    ['One-Handed Maces']    = 'Hitem:27901',    -- Blackout Truncheon
    ['One-Handed Swords']   = 'Hitem:28267',    -- Edge of the Cosmos
    ['Daggers']             = 'Hitem:30999',    -- Ashtongue Blade
    ['Fist Weapons']        = 'Hitem:27747',    -- Boggspine Knuckles
    ['Wands']               = 'Hitem:25640',    -- Nesingwary Safari Stick
    ['Two-Handed Axes']     = 'Hitem:32663',    -- Apexis Cleaver
    ['Two-Handed Maces']    = 'Hitem:30093',    -- Great Earthforged Hammer
    ['Two-Handed Swords']   = 'Hitem:27769',    -- Endbringer
    ['Polearms']            = 'Hitem:24044',    -- Hellreaver
    ['Staves']              = 'Hitem:25760',    -- Battle Mage's Baton
    ['Bows']                = 'Hitem:31072',    -- Lohn'goron, Bow of the Torn-heart
    ['Crossbows']           = 'Hitem:28397',    -- Emberhawk Crossbow
    ['Guns']                = 'Hitem:31000',    -- Bloodwarder's Rifle
    ['Shields']             = 'Hitem:31491',    -- Netherwing Defender's Shield
	['Cloth']				= 'Hitem:31216',	-- Abyssal Cloth Footwraps
    ['Leather']             = 'Hitem:31215',    -- Abyssal Leather Treads
    ['Mail']                = 'Hitem:31214',    -- Abyssal Mail Greaves
    ['Plate']               = 'Hitem:31213'     -- Abyssal Plate Sabatons
}
local localizeLootTypesCount = 0
local hasEnglishLocale = (GetLocale() == 'enUS')

function LootMaster:LocalizeLootTypes()

	if not self.bindingtooltip then
		self.bindingtooltip = CreateFrame("GameTooltip", "LootMasterBindingTooltip", UIParent, "GameTooltipTemplate")
	end
	local tip = self.bindingtooltip
	tip:SetOwner(UIParent, "ANCHOR_NONE")

    localizeLootTypesCount = localizeLootTypesCount + 1
    local failed = false;
    for sType, item in pairs(subTypeLocalizedLookup) do
        local itemName, _, _, _, _, _, itemSubType, _, _, _ = GetItemInfo(item)
        if itemName and itemSubType then
            -- Sanitycheck when we have an enUS client. Check our output against the strings we already know.
            if hasEnglishLocale and itemSubType~=sType then
                self:Print(format('Failed looking up localized strings for itemSubTypes. Expected %s for %s, got %s. Please report!', tostring(sType), tostring(item), tostring(itemSubType)))
            else
                subTypeLocalized[itemSubType] = sType
            end
            localizeLootTypesCount = 0
            subTypeLocalizedLookup[sType] = nil;
        else
            tip:SetHyperlink(item)
            failed = true
            -- stop the loop and try again after 2 secs
            break
        end
    end
    if failed then
		if localizeLootTypesCount<5 then
			self:ScheduleTimer("LocalizeLootTypes", 2)
		else
			self:Print(format('Failed looking up localized loottypes, bailing out permanently!'))
		end
	end
end

--[[
	Version check via AceComm
  ]]--
function LootMaster:CommVersionCheckRequest(prefix, message, distribution, sender)
	local _,_,lastCompVersion,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	lastCompVersion = tonumber(lastCompVersion) or 0
	senderVersionInt = tonumber(senderVersionInt) or 0
	if (debug) then
		self:Print( string.format("VReq from %s, has version %s", sender, senderVersionString) )
	end
	if (senderVersionInt<iVersion) then
		-- Senders version has been outdated
		self:SendCommMessage("CouncilLMVRsp", lastCompVersion .. "_" .. iVersion .. "_" .. version, "WHISPER", sender)
	end
	if (senderVersionInt>iVersion and not debug) then
		-- Our version is outdated
		if (lastCompVersion>iVersion) then
			self:ShowUrgentUpdateFrame( sender, iVersion, senderVersionString )
		else
			self:ShowUpdateFrame( sender, iVersion, senderVersionString )
		end
	end
end
function LootMaster:CommVersionCheckResponse(prefix, message, distribution, sender)
	local _,_,lastCompVersion,senderVersionInt, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	lastCompVersion = tonumber(lastCompVersion) or 0
	senderVersionInt = tonumber(senderVersionInt) or 0
	if (debug) then
		self:Print( string.format("VResp from %s, has version %s", sender, senderVersionString) )
	end
    if senderVersionInt~=0 and self.versioncheckframe and self.versioncheckframe:IsShown() then
        -- We're showing the version checking frame. lets update it
        local memberID = self.versioncheckframe.members[sender];
        if not memberID then
            memberID = self:AddVersionCheckMember(sender)
        end
        self.versioncheckframe.rows[memberID]["cols"][2].value=senderVersionString;
        self.versioncheckframe.rows[memberID]["cols"][3].value=senderVersionInt;
        if self.versioncheckframe.rows[memberID]["start"] then
            self.versioncheckframe.rows[memberID]["cols"][4].value=GetTime() - self.versioncheckframe.rows[memberID]["start"];
        end
        self.versioncheckframe.sstScroll:SetData( self.versioncheckframe.rows )
        self.versioncheckframe.sstScroll:SortData();
        self.versioncheckframe.sstScroll:DoFilter();
    end
	if (senderVersionInt>iVersion and not debug) then
		-- 	Our version is outdated
        if (lastCompVersion>iVersion) then
			self:ShowUrgentUpdateFrame( sender, iVersion, senderVersionString )
		else
			self:ShowUpdateFrame( sender, iVersion, senderVersionString )
		end
	end
end


--[DEBUG LOGGING STUFF]--
local debuglog = {}
function LootMaster:LogSize()
    return #debuglog
end
function LootMaster:RecurseLogOutput(entry)
    local output = {}
    local temp, itype
    for key, data in pairs(entry) do
        itype = type(data)
        if itype == 'table' then
            temp = self:RecurseLogOutput(data)
            itype = 't'
        elseif itype == 'number' then
            temp = data
            itype = 'i'
        else
            temp = format('"%s"',tostring(data))
        end
        tinsert(output, format('"%s":%s', key, temp))
    end
    return '{' .. strjoin(',', unpack(output)) .. '}'
end
function LootMaster:OutputLog()
    local output = self:RecurseLogOutput(debuglog);

    StaticPopupDialogs["COUNCIL_LOGOUTPUT_POPUP"] = {
        text = 'CouncilLootmaster: please copy/paste the text below and put it up on the guild forum if you want, along with an explanation of the situation.',
        button1 = nil,
        button2 = 'OK',
        timeout = 0,
        whileDead = 1,
        --exclusive = 0,
        --showAlert = 0,
        hideOnEscape = 1,
        hasEditBox = 1,
        maxLetters = 0,
        OnShow = function(self)
            self.editBox:SetText(output);
            self.editBox:SetFocus();
            self.editBox:HighlightText();
        end,
        OnHide = function(self)
            if ( ChatFrameEditBox:IsShown() ) then
                ChatFrameEditBox:SetFocus();
            end
            self.editBox:SetText("");
        end,
        EditBoxOnEscapePressed = function(self)
            self:GetParent():Hide();
            ClearCursor();
        end
    };
    StaticPopup_Show("COUNCIL_LOGOUTPUT_POPUP")
end
function LootMaster:CreateLogEntry()
    local entry = {
        ["ts"]   = GetTime()
    }
    tinsert(debuglog, entry);
    while #debuglog>50 do
        tremove(debuglog, 1);
    end
    return entry;
end
